<?php
/*
 * rh_exercise.php - Vips plugin for Stud.IP
 * Copyright (c) 2006-2008  Elmar Ludwig, Martin Schrder
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */

/* studip/vips/exercises/rh_exercise.php
 *
 * Zuordnungsaufgabe. Eine Liste von Texten, die einander zugeordnet werden mssen.
 */

require_once 'exercises/exercise.php';

class rh_exercise extends Exercise
{
    var $defaultArray;

    // constructor
    function rh_exercise($xml = null)
    {
        parent::Exercise($xml);

        $this->type = 'rh_exercise';

        if (!isset($xml)) {  // construct exercise using request parameters
            $this->defaultArray = $this->defaultArrayFromInput();
            $this->answerArray  = $this->answerArrayFromInput();
            $this->correctArray = $this->correctArrayFromInput();
            $this->numberOfAnswers = count($this->answerArray);
        }
        else {  // construct exercise using XML
            $dom_xml = $this->getDomXML($xml);
            $answers = $dom_xml->get_elements_by_tagname('Answer');

            foreach ($answers as $node)
            {
                $this->answerArray[] = utf8_decode($node->get_content());
                $this->correctArray[] = $node->get_attribute('Default');
            }

            $defaults = $dom_xml->get_elements_by_tagname('Default');

            foreach ($defaults as $default)
            {
                $this->defaultArray[] = utf8_decode($default->get_content());
            }

            $this->numberOfAnswers = count($this->answerArray);
        }
    }



    /**
     * Returns all variables which are peculiar for this exercise type.
     *
     * @returns An array with the key 'default_array'
     */
    function getQuirks()
    {
        return array(
            'default_array' => $this->defaultArray);
    }



    function setDefaultValues()
    {
        $this->numberOfAnswers = 5;
        $this->defaultArray = array(null, null, null, null, null);
        $this->answerArray  = array(null, null, null, null, null);
        $this->correctArray = array(   0,    1,    2,    3,    4);
    }



    /**
     * Computes points.
     */
    function evaluate($exercise, $solution)
    {
        $points = 0;

        $dom_xml = $this->getDomXML($exercise);
        $answers = $dom_xml->get_elements_by_tagname("Answer");

        $answer_array = array();
        $correct_array = array();

        foreach ($answers as $node)
        {
            $answer_array[] = utf8_decode($node->get_content());
            $correct_array[] = $node->get_attribute("Default");
        }

        $student_array = $this->getTagsFromXML($solution, "answer");

        for ($i = 0; $i < count($correct_array); $i++)
        {
            // if correct answer was given, give a point
            if ($correct_array[$i] == $student_array[$i])
            {
                $points++;
            }

            // don't subtract points if an answer was wrong
        }

        $m_safe = 0;
        if ($points <= count($answer_array) && $points >= 0)
        {
            $m_safe = 1;
        }

        $m_percent = $points * 100 / count($correct_array);

        $m_return[1] = $m_percent;
        $m_return[2] = $m_safe;
        $m_return[5] = $points;
        $m_return['percent'] = $m_percent;
        $m_return['safe']    = $m_safe;

        return $m_return;
    }



    /**
     * Creates XML to store student's solution.
     */
    function genSolution($p_exercise_id, $p_number_of_answers = null)
    {
        $i = 0;
        $xml = "<solution id=\"".$p_exercise_id."\">";

        foreach (vips_get_int_array_param('student_answer') as $answer)
        {
            $xml .= "<answer id=\"".$i++."\">";
            $xml .= "<body>".htmlspecialchars($answer)."</body>";
            $xml .= "</answer>";
        }

        $xml .= "</solution>";

        return vips_filter_xml_string($xml);
    }



    /**
     * Returns XML data representing the exercise, ready to be stored in the
     * database.
     *
     * @returns String The XML code representing the exercise.
     */
    function getExerciseConstruct()
    {
        $xml = '<TestItem ID="'.$this->type.'" Type="Zuordnung" Language="'.$this->language.'">';
        $xml .= '<Name>'.htmlspecialchars($this->name).'</Name>';
        $xml .= '<Hint>'.htmlspecialchars($this->hint).'</Hint>';
        $xml .= '<CommentField Visible="'.$this->commentAllowed.'"></CommentField>';
        $xml .= '<Question>';
        $xml .= '<Paragraph>'.$this->question.'</Paragraph>';
        $xml .= '</Question>';
        for ($i = 0; $i < count($this->defaultArray); $i++) {
            $xml .= '<Default ID="'.$i.'">'.htmlspecialchars($this->defaultArray[$i]).'</Default>';
        }
        for ($i = 0; $i < count($this->answerArray); $i++) {
            $xml .= '<Answer ID="'.$i.'" Default="'.$this->correctArray[$i].'">'.htmlspecialchars($this->answerArray[$i]).'</Answer>';
        }
        $xml .= '</TestItem>';



        // "special actions": add/delete pairs, mix/move answers //

        $special_action = false;

        if (vips_get_submit_param('add_pairs')) {
            $special_action = true;
            $number_to_add = vips_get_int_param('number_to_add');
            $xml = $this->add_pairs($xml, $number_to_add);
        }
        else if (vips_get_submit_param('mix')) {
            $special_action = true;
            $xml = $this->mix_answers($xml);
        }
        else {  // check if answers shall be moved or pairs shall be deleted
            for ($i = 0; $i < $this->numberOfAnswers; $i++) {
                if (vips_get_submit_param('move_up_answer_'.$i)) {
                    $special_action = true;
                    $xml = $this->move_answer($xml, $i, 'up');
                }
                else if (vips_get_submit_param('move_down_answer_'.$i)) {
                    $special_action = true;
                    $xml = $this->move_answer($xml, $i, 'down');
                }
                else if (vips_get_submit_param('delete_pair_'.$i)) {
                    $special_action = true;
                    $xml = $this->delete_pairs($xml, $i);
                }
            }
        }

        // if no special action was executed, ie. if "save" button has been pressed
        if ($special_action === false) {
            $xml = $this->delete_pairs($xml);  // delete all empty pairs
        }

        return vips_filter_xml_string($xml);
    }



    /**
     * Moves an answer up or down.
     */
    function move_answer($p_xml, $p_position, $p_direction)
    {
        $dom_doc = $this->getDomXML($p_xml);
        $answers = $dom_doc->get_elements_by_tagname("Answer");

        $node = $answers[$p_position];
        $parent = $node->parent_node();

        if ($p_direction == "up")
        {
            $prev_node = $node->previous_sibling();
            $prev_node->set_attribute("ID", $prev_node->get_attribute("ID") + 1);
            $node->set_attribute("ID", $node->get_attribute("ID") - 1);
            $parent->insert_before($node, $prev_node);
        }
        else if ($p_direction == "down")
        {
            $next_node = $node->next_sibling();
            $next_node->set_attribute("ID", $next_node->get_attribute("ID") - 1);
            $node->set_attribute("ID", $node->get_attribute("ID") + 1);
            $parent->insert_before($next_node, $node);
        }

        return utf8_decode($dom_doc->dump_mem());
    }



    /**
     * Adds a given number of empty pairs to XML.
     */
    function add_pairs($p_xml, $p_count)
    {
        $dom_doc = $this->getDomXML($p_xml);
        $defaults = $dom_doc->get_elements_by_tagname("Default");
        $answers = $dom_doc->get_elements_by_tagname("Answer");

        if (count($defaults) != count($answers))
        {
            // Something went wrong, as number of defaults is inequal to number of answers!
            return false;
        }

        $number_of_pairs = count($defaults);
        $root = $dom_doc->document_element();

        for ($i = 0; $i < $p_count; $i++)
        {
            $empty_default_node = $dom_doc->create_element("Default");
            $empty_answer_node = $dom_doc->create_element("Answer");

            $empty_default_node->append_child($dom_doc->create_attribute("ID", $i+$number_of_pairs));
            $empty_default_node->append_child($dom_doc->create_text_node(""));

            $empty_answer_node->append_child($dom_doc->create_attribute("ID", $i+$number_of_pairs));
            $empty_answer_node->append_child($dom_doc->create_attribute("Default", $i+$number_of_pairs));
            $empty_answer_node->append_child($dom_doc->create_text_node(""));

            $root->append_child($empty_default_node);
            $root->append_child($empty_answer_node);
        }

        $this->numberOfAnswers += $p_count;

        return utf8_decode($dom_doc->dump_mem());
    }



    /**
     * Randomizes order of answers.
     */
    function mix_answers($p_xml)
    {
        $dom_doc = $this->getDomXML($p_xml);
        $answers = $dom_doc->get_elements_by_tagname("Answer");

        $i = 0;
        foreach ($answers as $node) {
            $ids[] = $i++;
        }
        srand((float)microtime() * 1000000);
        shuffle($ids);

        foreach ($answers as $node) {
            $node->set_attribute("ID", array_shift($ids));
        }

        $answers = $this->sort_by_id($answers);

        return utf8_decode($dom_doc->dump_mem());
    }



    /**
     * Deletes pairs from XML.
     *
     * If you do not set <tt>$p_id</tt>, all unassigned pairs are deleted,
     * otherwise the <tt>Default</tt> with the id <tt>$p_id</tt> and the
     * corresponding <tt>Answer</tt> are removed.
     */
    function delete_pairs($p_xml, $p_id = false)
    {
        $dom_doc = $this->getDomXML($p_xml);
        $defaults = $dom_doc->get_elements_by_tagname("Default");
        $answers = $dom_doc->get_elements_by_tagname("Answer");

        if (count($defaults) != count($answers))
        {
            // Something went wrong, as number of defaults is inequal to number of answers!
            return false;
        }

        // delete only one pair
        if ($p_id !== false)
        {
            $parent = $defaults[$p_id]->parent_node();
            $parent->remove_child($defaults[$p_id]);

            foreach ($answers as $answer_node)
            {
                if ($answer_node->get_attribute("Default") == $p_id)
                {
                    $parent = $answer_node->parent_node();
                    $parent->remove_child($answer_node);
                    break;
                }
            }

            $this->numberOfAnswers -= 1;
        }

        // delete all empty pairs
        else
        {
            // check each default node
            foreach ($defaults as $default_node)
            {
                $id = $default_node->get_attribute("ID");

                // Is default node empty?
                if (isset($default_node) && $default_node->get_content() == "")
                {
                    // look for the corresponding answer node
                    foreach ($answers as $answer_node)
                    {
                        if (isset($answer_node) && $answer_node->get_attribute("Default") == $id)
                        {
                            // Corresponding node found... Is it empty?
                            if ($answer_node->get_content() == "")
                            {
                                // yes, it is empty...
                                $parent = $default_node->parent_node();
                                $parent->remove_child($default_node);

                                $parent = $answer_node->parent_node();
                                $parent->remove_child($answer_node);

                                $this->numberOfAnswers -= 1;
                            }
                        }
                    }
                }
            }
        }

        // keep ids in order and without "holes": 0, 1, 2, ..., n
        $this->reassign_ids
            ( $dom_doc->get_elements_by_tagname("Default")
            , $dom_doc->get_elements_by_tagname("Answer")
            );

        return utf8_decode($dom_doc->dump_mem());
    }



    /**
     * Checks a student's solution whether two or more answers are assigned to
     * one single default.
     */
    function checkSolutionXML($p_xml)
    {
        if (!isset($p_xml)) return 0;   // FIXME is this correct?

        $dom_doc = $this->getDomXML($p_xml);
        $answers = $dom_doc->get_elements_by_tagname("answer");
        $already_assigned = array();

        foreach ($answers as $answer)
        {
            $content = $answer->get_content();

            if ($content == "NULL") continue;  // skip NULL values

            // double entry found!
            if (array_search($content, $already_assigned) !== false)
            {
                $double_assigned = true;
            }
            array_push($already_assigned, $content);
        }

        if ($double_assigned) return 0;
        else return 1;
    }



    function sort_by_id($p_array)
    {
        // "Bubble Sort" :-)
        $i = count($p_array)-1;
        $first_node = $p_array[0];
        while ($i >= 0)
        {
            foreach ($p_array as $node)
            {
                // look for greatest id and put it to the front
                if ($node->get_attribute("ID") == $i)
                {
                    if ($node !== $first_node)
                    {
                        $parent = $node->parent_node();
                        $parent->insert_before($node, $first_node);
                        $first_node = $node;
                    }
                    $i--;
                }
            }
        }
        return $p_array;
    }



    function reassign_ids($p_defaults, $p_answers)
    {
        // for each default
        for ($i = 0; $i < count($p_defaults); $i++)
        {
            // is id not in order?
            if (($old_id = $p_defaults[$i]->get_attribute("ID")) != $i)
            {
                // correct the id
                $p_defaults[$i]->set_attribute("ID", $i);

                // find the corresponding answer
                foreach ($p_answers as $answer_node)
                {
                    if ($answer_node->get_attribute("Default") == $old_id)
                    {
                        // update answer's "Default" attribute
                        $answer_node->set_attribute("Default", $i);
                    }
                }
            }

            // set correct id for answer
            $p_answers[$i]->set_attribute("ID", $i);
        }
    }

    /**
     * Export this exercise to plain text format.
     */
    function exportText ()
    {
        $result = parent::exportText('ZU-Frage');

        for ($i = 0; $i < count($this->answerArray); ++$i)
        {
            $result .= 'Vorgabe: '.$this->defaultArray[$i]."\n";

            for ($j = 0; $j < count($this->correctArray); ++$j)
                if ($this->correctArray[$j] == $i)
                    $result .= 'Antwort: '.$this->answerArray[$j]."\n";
        }

        return $result;
    }



    /**
     * Creates a template for editing a rh_exercise.
     *
     * @returns The template for editing this exercise
     */
    function getEditTemplate()
    {
        global $vipsTemplateFactory,
            $g_outlet;

        $template = $vipsTemplateFactory->open('exercises/edit_rh_exercise');
        $template->set_attributes(array(
            'answer_array'      => $this->getAnswerArray(),
            'correct_array'     => $this->getCorrectArray(),
            'number_of_answers' => $this->getNumberOfAnswers(),
            'default_array'     => $this->defaultArray,
            'picturepath'       => $g_outlet->picturepath));

        return $template;
    }



    /**
     * Creates a template for solving an rh_exercise.
     *
     * @returns The template
     */
    function getSolveTemplate($solution_xml = null)
    {
        global $vipsTemplateFactory;

        if (isset($solution_xml)) {
            $student_array = $this->getTagsFromXML($solution_xml, 'answer');
        }

        $template = $vipsTemplateFactory->open('exercises/solve_rh_exercise');
        $template->set_attributes(array(
            'exercise_question' => $this->question,
            'exercise_hint'     => $this->hint,
            'default_array'     => $this->defaultArray,
            'answer_array'      => $this->answerArray,
            'student_array'     => $student_array));

        return $template;
    }



    function getWikiTemplate($solution_xml = null)
    {
        global $vipsTemplateFactory;

        if (isset($solution_xml)) {
            $student_array = $this->getTagsFromXML($solution_xml, 'answer');
        }

        $template = $vipsTemplateFactory->open('exercises/wiki_rh_exercise');
        $template->set_attributes(array(
            'name'          => $this->name,
            'question'      => $this->question,
            'hint'          => $this->hint,
            'default_array' => $this->defaultArray,
            'answer_array'  => $this->answerArray,
            'student_array' => $student_array
            ));

        return $template;
    }



    function getCorrectedWikiTemplate($solution_xml)
    {
        global $vipsTemplateFactory;

        $student_array = $this->getTagsFromXML($solution_xml, 'answer');

        $template = $vipsTemplateFactory->open('exercises/wiki_rh_exercise_corrected');
        $template->set_attributes(array(
            'name'          => $this->name,
            'question'      => $this->question,
            'hint'          => $this->hint,
            'default_array' => $this->defaultArray,
            'answer_array'  => $this->answerArray,
            'correct_array' => $this->correctArray,
            'student_array' => $student_array
            ));

        return $template;
    }



    function defaultArrayFromInput()
    {
        $default_array = array();

        for ($i = 0; $i < vips_get_int_param('number_of_answers'); $i++) {
            $default_array[] = vips_get_string_param('default_'.$i, true);
        }

        return $default_array;
    }



    function answerArrayFromInput()
    {
        $answer_array = array();

        for ($i = 0; $i < vips_get_int_param('number_of_answers'); $i++) {
            $answer_array[] = vips_get_string_param('answer_'.$i, true);
        }

        return $answer_array;
    }



    function correctArrayFromInput()
    {
        $correct_array = array();

        for ($i = 0; $i < vips_get_int_param('number_of_answers'); $i++) {
            $correct_array[] = vips_get_int_param('correct_'.$i);
        }

        return $correct_array;
    }

} // end class
