<?PHP
/*
 * vips_common.inc.php - Vips plugin for Stud.IP
 * Copyright (c) 2003-2005  Erik Schmitt, Philipp Hgelmeyer
 * Copyright (c) 2005-2006  Christa Deiwiks
 * Copyright (c) 2006-2008  Elmar Ludwig, Martin Schrder
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */

/**
 * This file contains classes and functions needed everywhere in the
 * vips code.
 * Several functions here were from former versions of vips but aren't used anymore but
 * I left them in here, might help...
 * @author: Phillip Huegelmeyer, Christa Deiwiks
 * @version: 2006-08-11
 */

define('UNUSED_GRADE', 101);

/**
 * Originally created to do more than htmlspecialchars()...
 * simple version of htmlReady() from visual.inc.php
 */
function html_ready ($string) {
    $string = htmlspecialchars($string);
    $string = preg_replace('/&amp;#(x[0-9a-f]+|[0-9]+);/i', '&#$1;', $string);
    return $string;
}

/**
 * Delete all characters outside the valid character range for XML
 * documents (#x9 | #xA | #xD | [#x20-#xD7FF] | [#xE000-#xFFFD]).
 */
function vips_filter_xml_string ($xml) {
    return preg_replace("/[^\t\n\r -\xFF]/", '', $xml);
}

/**
 * Return URL of Vips dispatcher script.
 * $params can contain optional additional parameters
 */
function vips_url ($params = array())
{
    global $vipsPlugin;

    return PluginEngine::getLink($vipsPlugin, $params);
}

/**
 * Return unencoded URL of Vips dispatcher script.
 * $params can contain optional additional parameters
 */
function vips_url_raw ($params = array())
{
    global $vipsPlugin;

    if (version_compare($GLOBALS['SOFTWARE_VERSION'], '1.7', '>=')) {
        return PluginEngine::getURL($vipsPlugin, $params);
    }

    return PluginEngine::getLink($vipsPlugin, $params);
}

/**
 * Unset all variables set by register_globals (if enabled).
 * Note: The session variables 'auth' and 'SessSemName' are preserved.
 */
function vips_unregister_globals ()
{
    if (!ini_get('register_globals')) {
        return;
    }

    if (isset($_REQUEST['GLOBALS']) || isset($_FILES['GLOBALS'])) {
        die('GLOBALS overwrite attempt detected');
    }

    $noUnset = array('GLOBALS', '_GET', '_POST', '_COOKIE',
                     '_REQUEST', '_SERVER', '_ENV', '_FILES',
                     'auth', 'SessSemName');
    $vars = array_merge($_GET, $_POST, $_COOKIE, $_SERVER, $_ENV, $_FILES);

    if (isset($_SESSION)) {
        $vars = array_merge($vars, $_SESSION);
    }

    foreach ($vars as $var => $value) {
        if (!in_array($var, $noUnset) && isset($GLOBALS[$var])) {
            unset($GLOBALS[$var]);
        }
    }
}

/**
 * Cuts a double after specified kommastellen, e.g.:
 * roundLikeFloor(2.6318,3) => 2.631
 * roundLikeFloor(2.6315,0) => 2
 *
 * @param: double $double Double to round
 * @param: int $kommastelle Number of kommastellen to be retained
 * @return: double $double The cut-off double
 * @author: cdeiwiks
 */
function roundLikeFloor($double, $kommastelle) {
        $factor = pow(10,$kommastelle);
        $double = $double*$factor;
        $double = floor($double);
        $double = $double/$factor;
        return $double;
}

/**
 * Returns a normalized version of a string (see comments within function)
 *
 * @param: String $string String to be normalized
 * @return: The normalized string
 * @author: cdeiwiks
 */
function normalizeText ($string)
{
        // compress white space
        $string = preg_replace('/\\s+/', ' ', $string);

        // delete blanks before and after [](){}:,;.!?'"=*/+-
        $string = preg_replace('/ *([][(){}:,;.!?\'"=*\\/+-]) */', '$1', $string);

        // convert to lower case
        return strtolower($string);
}

/**
 * Looks up a parameter in the post and get environment and returns it. Its type
 * will be string.
 *
 * @param $param The name of the parameter which is looked up
 * @returns The value as string (if found), else NULL
 */
function vips_get_param ($param)
{
    if (isset($_POST[$param])) {  // search post environment
        $value = $_POST[$param];
    }
    else if (isset($_GET[$param])) {  // search get environment
        $value = $_GET[$param];
    }
    else {  // variable is not set
        $value = null;
    }

    if (isset($value) && is_array($value)) {  // assert that $value is not an array
        $value = array_shift($value);
    }

    return $value;
}



/**
 * Looks up a parameter in the post and get environment and returns it as
 * integer.
 *
 * @param $param The name of the parameter which is looked up
 * @returns The value as integer (if found), else NULL
 */
function vips_get_int_param ($param)
{
    $value = vips_get_param($param);

    if (isset($value)) {
        $value = (int) $value;
    }

    return $value;
}



/**
 * Looks up a parameter in the post and get environment and returns it as float.
 *
 * @param $param The name of the parameter which is looked up
 * @returns The value as float (if found), else NULL
 */
function vips_get_float_param ($param)
{
    $value = vips_get_param($param);

    if (isset($value)) {
        $value = (float) strtr($value, ',', '.');
    }

    return $value;
}



/**
 * Looks up a parameter in the post and get environment and returns it as
 * string. The contents of the string is quoted with addslashes().
 *
 * @param $param The name of the parameter which is looked up
 * @param $trim  trim whitespace at start and end of value (optional)
 * @returns The value as string (if found), else NULL
 */
function vips_get_string_param ($param, $trim = false)
{
    $value = vips_get_param($param);

    if (isset($value) && !get_magic_quotes_gpc()) {
        $value = addslashes($value);
    }

    if (isset($value) && $trim) {
        $value = trim($value);
    }

    return $value;
}



/**
 * Looks up a parameter in the post and get environment and returns it as
 * string. The contents of the string is not quoted.
 *
 * @param $param The name of the parameter which is looked up
 * @param $trim  trim whitespace at start and end of value (optional)
 * @returns The value as string (if found), else NULL
 */
function vips_get_unquoted_string_param ($param, $trim = false)
{
    $value = vips_get_param($param);

    if (isset($value) && get_magic_quotes_gpc()) {
        $value = stripslashes($value);
    }

    if (isset($value) && $trim) {
        $value = trim($value);
    }

    return $value;
}



/**
 * Looks up a parameter in the post and get environment and returns it as
 * alphanumeric string.
 *
 * @param $param The name of the parameter which is looked up
 * @returns The value as alphanumeric string (if found), else NULL
 */
function vips_get_option_param ($param)
{
    $value = vips_get_param($param);

    if (isset($value) && preg_match('/\\W/', $value)) {
        $value = '';
    }

    return $value;
}



/**
 * Looks up a parameter array in the post and get environment and returns it.
 * All elements' type will be string.
 *
 * @param $param The name of the parameter array which is looked up
 * @returns The array with string values (if found), else an empty array
 */
function vips_get_array_param ($param)
{
    if (isset($_POST[$param])) {  // search the post environment
        $array = $_POST[$param];
    }
    else if (isset($_GET[$param])) {  // search the get environment
        $array = $_GET[$param];
    }
    else {  // variable is not set
        $array = array();
    }

    if (isset($array) && !is_array($array)) {  // assert that $array is an array
        $array = array($array);
    }

    return $array;
}



/**
 * Looks up a parameter array in the post and get environment and returns it
 * with all elements castet to integer type.
 *
 * @param $param The name of the parameter array which is looked up
 * @returns The array with integer values (if found), else an empty array
 */
function vips_get_int_array_param ($param)
{
    $array = vips_get_array_param($param);

    if (isset($array)) {
        foreach ($array as $key => $value) {
            $array[$key] = (int) $value;
        }
    }

    return $array;
}



/**
 * Looks up a parameter array in the post and get environment and returns it
 * with all elements casted to float type.
 *
 * @param $param The name of the parameter array which is looked up
 * @returns The array with float values (if found), else an empty array
 */
function vips_get_float_array_param ($param)
{
    $array = vips_get_array_param($param);

    if (isset($array)) {
        foreach ($array as $key => $value) {
            $array[$key] = (float) strtr($value, ',', '.');
        }
    }

    return $array;
}



/**
 * Looks up a parameter array in the post and get environment and returns it
 * with all elements casted to string. The contents of each element is quoted
 * with addslashes().
 *
 * @param $param The name of the parameter array which is looked up
 * @param $trim  trim whitespace at start and end of each value (optional)
 * @returns The array with string values (if found), else an empty array
 */
function vips_get_string_array_param ($param, $trim = false)
{
    $array = vips_get_array_param($param);

    if (isset($array) && !get_magic_quotes_gpc()) {
        foreach ($array as $key => $value) {
            $array[$key] = addslashes($value);
        }
    }

    if (isset($array) && $trim) {
        foreach ($array as $key => $value) {
            $array[$key] = trim($value);
        }
    }

    return $array;
}



/**
 * Looks up a parameter array in the post and get environment and returns it
 * with all elements casted to string. The contents of each element is not
 * quoted.
 *
 * @param $param The name of the parameter array which is looked up
 * @param $trim  trim whitespace at start and end of each value (optional)
 * @returns The array with string values (if found), else an empty array
 */
function vips_get_unquoted_string_array_param ($param, $trim = false)
{
    $array = vips_get_array_param($param);

    if (isset($array) && get_magic_quotes_gpc()) {
        foreach ($array as $key => $value) {
            $array[$key] = stripslashes($value);
        }
    }

    if (isset($array) && $trim) {
        foreach ($array as $key => $value) {
            $array[$key] = trim($value);
        }
    }

    return $array;
}



/**
 * Looks up a parameter array in the post and get environment and returns it
 * with all elements as alphanumeric strings.
 *
 * @param $param The name of the parameter array which is looked up
 * @returns The array with alphanumeric strings (if found), else an empty array
 */
function vips_get_option_array_param ($param)
{
    $array = vips_get_array_param($param);

    if (isset($array)) {
        foreach ($array as $key => $value) {
            if (preg_match('/\\W/', $value)) {
                $array[$key] = '';
            }
        }
    }

    return $array;
}



/**
 * Checks whether a graphical submit button has been pressed.
 *
 * @param $param The name of the button
 * @returns True if the button has been pressed, else false
 */
function vips_get_submit_param ($param)
{
    $p  = vips_get_param($param);
    $px = vips_get_param($param.'_x');

    return isset($p) || isset($px);
}



/**
 * Function returns the maximum number of points a person or a group
 * can get for a given exercise sheet.
 *
 * @param integer the id of the exercise sheet
 * @param string the type of the exercise sheet
 * @return integer the number of maximal points for this exercise sheet
 */
function get_max_points_assignment($p_exercise_sheet_id, $p_exercise_sheet_type) {
        global $vips_db;

        switch ($p_exercise_sheet_type) {
            case "vips_klausur":
                $m_insheetType = "vips_inKlausur";
                break;
            case "vips_uebungsblatt":
                $m_insheetType = "vips_inUebungsblatt";
                break;
            case "vips_gruppe":
                $m_insheetType = "vips_inGruppe";
                break;
        }
        $query = "SELECT SUM(Punkte) AS points FROM ".$m_insheetType." WHERE ".$p_exercise_sheet_type."='".$p_exercise_sheet_id."'";
        $result = $vips_db->executeQuery($query);
        $result->next();
        $m_points = $result->getFloat('points');
        if ($m_points == NULL) {
            $m_points = 0;
        }
        return $m_points;
}

/**
 * Returns the full name of a user (without title) or group.
 *
 * @param  string $solver_id user or group id of solver
 * @return string full name of user or name of group (or NULL)
 */
function get_solver_fullname ($solver_id)
{
    global $vips_db;

    if (strlen($solver_id) == 32) {
        $user = new StudIPUser();
        $user->setUserid($solver_id);
        return $user->getGivenname().' '.$user->getSurname();
    }

    $result = $vips_db->executeQuery("SELECT * FROM vips_gruppe WHERE Gruppenid = $solver_id");

    if ($result->next()) {
        return $result->get('Gruppenname');
    } else {
        return NULL;
    }
}

/**
 * Returns the name of a user (login name) or group.
 *
 * @param  string $solver_id user or group id of solver
 * @return string login name of user or name of group (or NULL)
 */
function get_solver_name ($solver_id)
{
    global $vips_db;

    if (strlen($solver_id) == 32) {
        $user = new StudIPUser();
        $user->setUserid($solver_id);
        return $user->getUsername();
    }

    $result = $vips_db->executeQuery("SELECT * FROM vips_gruppe WHERE Gruppenid = $solver_id");

    if ($result->next()) {
        return $result->get('Gruppenname');
    } else {
        return NULL;
    }
}

/**
 * Returns the student id for the given user id.
 */
function get_student_id ($user_id)
{
    static $stud_id;

    if (version_compare($GLOBALS['SOFTWARE_VERSION'], '1.6', '>=')) {
        require_once 'lib/classes/DataFieldEntry.class.php';
    } else {
        require_once 'lib/classes/DataFields.class.php';
    }

    if (!isset($stud_id)) {
        $config = new Config();
        $stud_id = $config->getValue('VIPS_STUDENT_ID_DATAFIELD');
    }

    if (version_compare($GLOBALS['SOFTWARE_VERSION'], '1.6', '>=')) {
        $fields = DataFieldEntry::getDataFieldEntries($user_id);
        $result = isset($fields[$stud_id]) ? $fields[$stud_id]->getValue() : NULL;
    } else {
        $fields = new DataFields($user_id);
        $fields = $fields->getLocalFields();
        $result = $fields[$stud_id]['content'];
    }

    return $result;
}

/**
 * Rounds a float value to the nearest half point, ie x.0 or x.5.
 *
 * @param float $points
 * @returns float The result.
 *
 * @author Martin Schrder
 * @date 2007-05-31
 */
function round_to_half_point($points)
{
    return round($points * 2) / 2;
}

/**
 * Discard any output buffered since last ob_start()
 */
function discard_buffer ($buffer)
{
    return '';
}

/**
 * Test whether editable solutions should be enabled.
 */
function vips_has_editable_solutions ()
{
    return version_compare($GLOBALS['SOFTWARE_VERSION'], '1.6', '>=') ||
           $GLOBALS['SOFTWARE_VERSION'] == 'Stud.IP 1.1.5-OS-1';
}

/**
 * Test whether AJAX functions should be enabled.
 */
function vips_use_ajax ()
{
    global $auth, $SOFTWARE_VERSION;

    return $auth->auth['jscript'] &&
           (version_compare($SOFTWARE_VERSION, '1.6', '>=') ||
            $SOFTWARE_VERSION == 'Stud.IP 1.1.5-OS-1');
}

/**
 * Clear default Stud.IP page layout and start with empty page.
 */
function vips_output_begin ($style = 'vips')
{
    global $vipsPlugin, $_include_additional_header;

    if ($style == 'print') {
        $css = $vipsPlugin->getPluginPath().'/css/vips_print.css';
    } else {
        $css = PluginEngine::getLink($vipsPlugin, array(), 'css');
    }

    $_include_additional_header = '<link rel="stylesheet" href="'.$css.'" type="text/css">';

    // this hack is necessary to disable the standard Stud.IP layout
    ob_end_clean();

    // note: second include of 'html_head.inc.php' fails on Stud.IP 1.2-os1
    if (function_exists('html_head')) {
        echo html_head();
    } else if (file_exists('html_head.inc.php')) {
        include 'html_head.inc.php';
    } else {
        include 'lib/include/html_head.inc.php';
    }
}

/**
 * Finish Vips output, everything printed hereafter will be dropped.
 */
function vips_output_end ()
{
    if (file_exists('html_end.inc.php')) {
        include 'html_end.inc.php';
    } else {
        include 'lib/include/html_end.inc.php';
    }

    // this hack is necessary to disable the standard Stud.IP layout
    ob_start('discard_buffer');
}

/**
 * NOT USED
 * Function returns the reached number of points of a person or a group
 * for a given exercise sheet.
 *
 * @param integer the id of the exercise sheet
 * @param string the type of the exercise sheet
 * @return integer the number of maximal points for this exercise sheet
 */
/*
function get_reached_points_assignment($p_exercise_sheet_id, $p_exercise_sheet_type, $p_userOrgroupID) {
        global $vips_db;

        if ($p_exercise_sheet_type == "vips_klausur")
        {
                // this query is correct but *VERY*(!!!) slow
//              $query = "SELECT SUM(`vips_K-Loesung-Punkte`.Punkte) AS ReachedPoints
//                      FROM `vips_K-Loesung`
//                              , `vips_K-Loesung-Punkte`
//                              , `vips_inKlausur`
//                      WHERE `vips_K-Loesung`.Loesungsid = `vips_K-Loesung-Punkte`.idLoesung
//                              AND `vips_K-Loesung`.Aufgabenid = `vips_inKlausur`.Aufgabe
//                              AND `vips_K-Loesung`.Klausurid = '$p_exercise_sheet_id'
//                              AND `vips_K-Loesung`.Userid = '$p_userOrgroupID'
//                              AND `vips_K-Loesung`.isLastSolution = 't'
//                      ";

                // this query is almost always correct (but counts also exercises which WERE in an
                // exercise sheet, were solved by students and have been removed afterwards) and
                // much quicker

                #$query = "SELECT DISTINCT SUM(`vips_K-Loesung-Punkte`.Punkte) AS ReachedPoints
                #       FROM `vips_K-Loesung`
                #               , `vips_K-Loesung-Punkte`
                #       WHERE `vips_K-Loesung`.Loesungsid = `vips_K-Loesung-Punkte`.idLoesung
                #               AND `vips_K-Loesung`.Klausurid = '$p_exercise_sheet_id'
                #               AND `vips_K-Loesung`.Userid = '$p_userOrgroupID'
                #               AND `vips_K-Loesung`.isLastSolution = 't'
                #       ";
                $query = "SELECT DISTINCT Aufgabenid, `vips_K-Loesung-Punkte`.Punkte "
                        ."FROM `vips_K-Loesung` "
                        ."    , `vips_K-Loesung-Punkte` "
                        ."    , `vips_inKlausur` "
                        ."WHERE `vips_K-Loesung`.Loesungsid = `vips_K-Loesung-Punkte`.idLoesung "
                        ."    AND `vips_K-Loesung`.Aufgabenid = `vips_inKlausur`.vips_aufgabe "
                        ."    AND `vips_K-Loesung`.Klausurid = '".$p_exercise_sheet_id."' "
                        ."    AND `vips_K-Loesung`.Userid = '".$p_userOrgroupID."' "
                        ."    AND `vips_K-Loesung`.isLastSolution = 't' "
                        ;
                $result = $vips_db->executeQuery($query);

                // Sorry, but MySQL cannot compute a SUM in connection with DISTINCT.
                while ($result->next())
                {
                        $m_reached_points += $result->get("Punkte");
                }

                return $m_reached_points;
        }
        else
        {
//              $query = "SELECT SUM(`vips_T-Loesung-Punkte`.Punkte) AS ReachedPoints
//                      FROM `vips_T-Loesung`
//                              , `vips_T-Loesung-Punkte`
//                              , `inUebungsblatt`
//                      WHERE `vips_T-Loesung`.Uebungsblattid = '$p_exercise_sheet_id'
//                              AND `vips_T-Loesung`.Gruppenid = '$p_userOrgroupID'
//                              AND `vips_T-Loesung`.isLastSolution = 't'
//                              AND `vips_T-Loesung`.Aufgabenid = `inUebungsblatt`.Aufgabe
//                              AND `vips_T-Loesung`.Loesungsid = `vips_T-Loesung-Punkte`.idLoesung
//                      ";
//
                $query = "SELECT SUM(`vips_T-Loesung-Punkte`.Punkte) AS ReachedPoints
                        FROM `vips_T-Loesung`
                                , `vips_T-Loesung-Punkte`
                        WHERE `vips_T-Loesung`.Loesungsid = `vips_T-Loesung-Punkte`.idLoesung
                                AND `vips_T-Loesung`.Uebungsblattid = '$p_exercise_sheet_id'
                                AND `vips_T-Loesung`.Gruppenid = '$p_userOrgroupID'
                                AND `vips_T-Loesung`.isLastSolution = 't'
                        ";
        }
        $result = $vips_db->executeQuery($query);
        $result->next();
        return($result->get("ReachedPoints"));
}
*/



/**
 * Fetches the solution of a given user to an exercise determined by position,
 * assignment and assignment type.
 *
 * @params $user_id           The md5 user id
 *         $exercise_position The exercise position in the sheet
 *         $assignment_id     The assignment id
 *         $assignment_type   The assignment type, one of "vips_klausur" or
 *                            "vips_uebungsblatt"
 * @returns The solution XML
 */
function getUserSolution($user_id, $exercise_position, $assignment_id, $assignment_type)
{
    global $vips_db;

    if ($assignment_type == 'vips_klausur') {
        $solution_table = 'vips_K_Loesung';
        $in_sheet_table = 'vips_inKlausur';
    }
    else {
        $solution_table = 'vips_T_Loesung';
        $in_sheet_table = 'vips_inUebungsblatt';
    }

    // check whether user is member of a group
    $group_id = null;
    if ($assignment_type == 'vips_uebungsblatt') {
        $sql = "SELECT vips_inGruppe.group_id
            FROM $assignment_type, vips_gruppe, vips_inGruppe
            WHERE $assignment_type.ID   = '$assignment_id'
              AND $assignment_type.Kurs = vips_gruppe.Kursid
              AND vips_gruppe.Gruppenid = vips_inGruppe.group_id
              AND vips_inGruppe.user_id = '$user_id'
              AND vips_inGruppe.end     = 'NULL'";
        $result = $vips_db->executeQuery($sql);
        if ($result->next()) {
            $group_id = $result->getInt('group_id');
        }
    }

    // fetch solution
    if (isset($group_id)) {  // group solution
        $sql = "SELECT Loesung
            FROM $in_sheet_table, $solution_table
            WHERE $in_sheet_table.Position = '$exercise_position'
              AND $in_sheet_table.vips_aufgabe = $solution_table.Aufgabenid
              AND $solution_table.Gruppenid = '$group_id'
              AND $solution_table.Assignmentid = '$assignment_id'
              AND $solution_table.isLastSolution = 't'";
    }
    else {  // user solution
        $sql = "SELECT Loesung
            FROM $in_sheet_table, $solution_table
            WHERE $in_sheet_table.Position = '$exercise_position'
              AND $in_sheet_table.vips_aufgabe = $solution_table.Aufgabenid
              AND $solution_table.Userid = '$user_id'
              AND $solution_table.Assignmentid = '$assignment_id'
              AND $solution_table.isLastSolution = 't'";
    }
    $result = $vips_db->executeQuery($sql);
    if ($result->next()) {
        $solution_xml = $result->getString('Loesung');
    }

    return $solution_xml;
}

?>
